      SUBROUTINE f7args(maxflg, flgi, flga, narg, arg, flg, istat)
c
c Load flags and arguments from the command line for a FORTRAN program.
c
c File f7args.f (Version 1.1).  Last modified 17:53:18, 6/7/95.
c
c.......................................................................
c
c From:  National Simulation Resource 
c        Center for Bioengineering
c        University of Washington
c        Box 357962
c        Seattle, WA 98195-7962
c
c        Dr. J. B. Bassingthwaighte, Director
c
c.......................................................................
c
c Copyright (C) 1995 by National Simulation Resource, Univ of WA.
c All Rights Reserved.
c 
c Software may be copied so long as this copyright notice is included.
c This software was developed with support from NIH grant RR-01243.
c Please cite this grant in any publication for which this software
c is used and send one reprint to the address given above.
c
c.......................................................................
c
c SYNOPSIS:
c
c     SUBROUTINE f7args(maxflg, flgi, flga, narg, arg, flg, istat)
c
c     CHARACTER*(*) flgi(*), flga(*), arg(*), flg(*)
c     INTEGER       maxflg, narg, istat
c
c.......................................................................
c
c DESCRIPTION:
c
c Load flags and arguments from the command line for a FORTRAN program.
c
c In this description, the phrase "command-line arguments" refers to
c blank-delimited strings on the command line that invoked a program.  
c Thus command-line arguments include the name of the program,
c command-line argument zero.
c
c The term "flag" refers to a command-line argument that starts with
c the character "-".  Flags are one or more characters in length. Thus,
c the single character "-" is a valid flag.
c
c The term "argument" refers to a command-line argument that is not the
c program name or  a flag.  An argument may be a "flag-associated
c argument" or a "stand-alone argument" (i.e. not associated with a
c flag).
c
c The calling function specifies a list of flags and argument indicators
c in the character arrays flgi and flga.  These vectors must have the
c same number of elements (maxflg).
c
c f7args will handle the following combinations of flags and arguments:
c 
c Flag/argument combination             flgi          flga
c ---------------------------------     ---------     -------------
c A flag without an argument:           Flag name     'NONE'
c A flag with an optional argument:     Flag name     'OPT'
c A flag with a mandatory argument:     Flag name     ' '
c A stand-alone argument:               'NONE'        ' '
c
c The flag names are case sensitive, but the strings 'NONE' and 'OPT'
c are case insensitive.
c
c When f7args has been used to load command-line arguments into the flg
c and arg arrays, f7flag and f7arg can be used to check for the presence
c and/or get the values of specific flags and arguments.
c 
c.......................................................................
c
c FORMAL PARAMETERS:
c
c INPUTS:
c
c    Name   Description
c    ------ ------------------------------------------------------------
c    maxflg The maximum # of elements in flgi, flga, arg, and flg.
c           maxflg = max. # of flags + max. # of stand-alone arguments
c    flgi   A character array of maxflg flag names. The flag name
c           includes the leading '-' character.
c    flga   A character array of maxflg argument indicators.
c
c OUTPUTS:
c
c    Name   Description
c    ------ ------------------------------------------------------------
c    narg   The number of elements loaded in arg and flg.
c           narg = # of command-line flags
c                  + # of command-line stand-alone arguments
c    arg    A character array of the flag-associated and stand-alone
c           arguments from the command line.
c           Dimensioned at least maxflg in the calling routine.
c    flg    A character array of the flags from the command line.
c           Dimensioned at least maxflg in the calling routine.
c    istat  Exit status (See DIAGNOSTICS)
c
c.......................................................................
c
c LIMITATIONS/WARNINGS:
c
c The maximum length of a command-line argument is 128 characters.
c
c Be careful in the use of optional stand-alone arguments.  A command
c with syntax:
c
c cmd_name [file_in] file_out
c
c presents no problems.  If there is one stand-alone argument, it is
c file_out.  If there are two, they are file_in and file_out
c respectively.  Consider, however, the syntax:
c
c cmd_name [file_in] [error_file] file_out
c
c One or three stand-alone arguments on the command line present no
c problem, but two lead to ambiguity.  The second is file_out, but
c the the usage of the first cannot be determined.
c
c.......................................................................
c
c DIAGNOSTICS:
c
c The exit status is returned in istat which can have the values:
c      0 - Normal exit.
c     -1 - Too many command-line arguments.
c     -2 - The command line contains an invalid flag.
c     -3 - A mandatory flag-associated argument is missing.
c     -4 - The command line contains too many stand-alone arguments.
c    -99 - The length of a command-line argument exceeds the length of
c          the strings in arg.
c
c.......................................................................
c
c EXAMPLES:
c
c Use f7args to load flags and arguments of a command-line with the
c syntax:
c
c cmd_name [-a [argA]] [-b] [-c argC] [file_in] file_out
c
c The value of maxflg should be 5, and the arrays flgi and flga should
c contain:
c
c         i   flgi(i) flga(i)
c        ---  ------- -------
c         1    '-a'    'opt'
c         2    '-b'    'none'
c         3    '-c'    '    '
c         4    'none'  '    '
c         5    'none'  '    '
c
c The command line: cmd_name -a -c abc file_name
c
c returns narg=3, istat=0 and flg and arg of:
c
c         i    flg(i)     arg(i)
c        ---  ------- -------------
c         1    '-a'    'NONE'
c         2    '-c'    'abc'
c         3    'NONE'  'file_name'
c
c The command line: cmd_name -a arg4a -b file_name1 file_name2
c
c returns narg=4, istat=0 and flg and arg of:
c
c         i    flg(i)     arg(i)
c        ---  ------- -------------
c         1    '-a'    'arg4a'
c         2    '-b'    'NONE'
c         3    'NONE'  'file_name1'
c         4    'NONE'  'file_name2'
c 
c The following command lines return error conditions:
c
c Command                           istat  Error
c --------------------------------  -----  -----------------------------
c cmd_name -a a1 -b -c a2 a3 f2 f3   -1    Too many arguments
c cmd_name -d a1 f_out               -2    -d is an invalid flag
c cmd_name -c -a arg1 f_out          -3    Argument for -c is missing
c cmd_name -c arg1 arg2 f_in f_out   -4    Too many arguments w/o flags
c
c.......................................................................
c
c SUBROUTINES/FUNCTIONS CALLED:
c
c NSR combined library:
c      f7argc: Get the total number of command-line arguments.
c      f7argv: Get the value of a command-line argument.
c      f7case: Return the upper case version of a string.
c
c.......................................................................
c
c SEE ALSO:
c
c      f7flag(3) - Get the value of a flag-associated argument.
c      f7arg(3)  - Get the value of a stand-alone argument.
c
c.......................................................................
c
c FILES:
c
c /usr/local/lib/libnsr.a       - library archive
c ~libnsr/lib/libmachdep/f7args - source files
c
c.......................................................................
c
c AUTHOR:
c
c National Simulation Resource
c Center for Bioengineering
c University of Washington
c Box 357962
c Seattle, WA 98195-7962
c
c.......................................................................
c
c FOR ASSISTANCE:
c
c Questions regarding this software can be sent by electronic mail to:
c   librarian@nsr.bioeng.washington.edu
c
c.......................................................................
c
c HISTORY:
c
c Written:  R. B. King (MAR95)
c
c Modified:
c Ver. N.N: Description (AUTHOR - DATE)
c
c-----------------------------------------------------------------------
c
c Data dictionary:
c
c cmdarg: value of current command-line argument
c i:      general loop counter
c iarg:   index of current command-line argument
c iloc:   pointer into flgi and flga vectors
c maxca:  length of strings in arg
c nargc:  # of command-line arguments
c nargcm: max. # of command-line arguments
c         = max. # of stand alone args + max. # of flags w/o args
c           + 2*(max. # of flags w/ optional args)
c           + 2*(max. # of flags w/ mandatory args)
c noflgf: # of stand-alone arguments found on the command-line
c noflgi: max. # of stand-alone arguments
c
c.......................................................................
c
      CHARACTER*(*) flgi(*), flga(*), arg(*), flg(*)
      INTEGER       maxflg, narg, istat
c
      INTEGER       iarg, noflgi, noflgf, nargcm, maxca, i, iloc, nargc
      INTEGER       f7argc, LNBLNK
      CHARACTER*128 cmdarg, f7case
      CHARACTER*63  sid1, sid2
c
      EXTERNAL      f7case, f7argc, f7argv
c
      DATA         sid1
     + /'@(#)f7args.f	1.1 created on 6/7/95 at 17:53:18.'/
      DATA         sid2
     + /'@(#)    Retrieved on 10/21/96 at 14:14:02.'/
c
c Initialize variables.
c
      maxca  = LEN(arg(1))
      iarg   = 1
      narg   = 0
      noflgf = 0
      noflgi = 0
      nargcm = 0
c
c     Set noflgi and nargcm.
c
      DO 1 i = 1, maxflg
          IF      (f7case(flgi(i)) .EQ. 'NONE') THEN
              nargcm = nargcm + 1
              noflgi = noflgi + 1
          ELSE IF (f7case(flga(i)) .EQ. 'OPT' ) THEN
              nargcm = nargcm + 2
          ELSE IF (f7case(flga(i)) .EQ. 'NONE' ) THEN
              nargcm = nargcm + 1
          ELSE
              nargcm = nargcm + 2
          END IF
    1 CONTINUE
c
c Get the number of command-line arguments.
c
      nargc = f7argc()
      IF (nargc .LT. 1) THEN
c
c         No command-line arguments.
          istat = 0
          RETURN
      ELSE IF (nargc .GT. nargcm) THEN
c
c         Too many command-line arguments.
          istat = -1
          RETURN
      END IF
c
c Check argument counts.
c
   99 IF (iarg .GT. nargc) THEN
c
c         Everything is OK.
          istat = 0
          RETURN
      END IF
c
      IF (iarg.GT.nargc .OR. narg.GT.maxflg) THEN
c
c         Argument counts exceeded.
          istat = -1
          RETURN
      END IF
c
c Get the next command-line argument.
c
      CALL f7argv(iarg, cmdarg)
      iarg = iarg + 1
c
c Check the command-line argument type.
c
      IF (cmdarg(1:1) .EQ. '-') THEN
c
c         The command-line argument is a flag.
c
c         Is it a legal flag?
c
          iloc = 0
          DO 2 i = 1, maxflg
              IF (flgi(i) .EQ. cmdarg) iloc = i
    2     CONTINUE
          IF (iloc .LT. 1) THEN
c
c             Not a legal flag.
c
              istat = -2
              RETURN
c
          ELSE
c
c             Legal flag.
c
c             Store it.
c
              narg = narg + 1
              flg(narg) = flgi(iloc)
c
c             Get the flag-associated argument.
c
c             Handle an optional flag-associated argument.
c
              IF      (f7case(flga(iloc)) .EQ. 'OPT' ) THEN
c
c                 No more command-line arguments.
c
                  IF (iarg .GT. nargc) THEN
                      arg(narg) = 'NONE'
c
c                 More command-line arguments.
c
                  ELSE
                      CALL f7argv(iarg, cmdarg)
                      IF (cmdarg(1:1) .EQ. '-') THEN
c
c                         The next command-line argument is a flag.
c
                          arg(narg) = 'NONE'
                      ELSE
c
c                         Found an optional flag-associated argument.
c                         Increment the argument count and store it
c                         unless it's too long.
c
                          iarg = iarg + 1
                          IF (LNBLNK(cmdarg) .GT. maxca) THEN
                              istat = -99
                              RETURN
                          END IF
                          arg(narg) = cmdarg(1:LNBLNK(cmdarg))
                      END IF
                  END IF
c
              ELSE IF (f7case(flga(iloc)) .NE. 'NONE') THEN
c
c                 Handle a mandatory flag-associated argument.
c
                  IF (iarg .GT. nargc) THEN
c
c                     No more command-line arguments.
c
                      istat = -3
                      RETURN
                  ELSE
c
c                     Get the flag-associated argument.
c
                      CALL f7argv(iarg, cmdarg)
c
                      IF (cmdarg(1:1) .EQ. '-') THEN
c
c                         The flag-associated argument is missing.
c
                          istat = -3
                          RETURN
                      ELSE
c
c                         Store the it unless it's too long.
c
                          iarg = iarg + 1
                          IF (LNBLNK(cmdarg) .GT. maxca) THEN
                              istat = -99
                              RETURN
                          END IF
                          arg(narg) = cmdarg(1:LNBLNK(cmdarg))
                      END IF
                  END IF
c
              ELSE
c
c                 Handle a flag without an associated argument.
c
                  arg(narg) = 'NONE'
              END IF
c
          END IF
c
      ELSE
c
c         The command-line argument is a stand-alone argument.
c
          IF (noflgf .GE. noflgi) THEN
c
c             No room to store it.
c
              istat = -4
              RETURN
c
          ELSE
c
c             Store the stand-alone argument unless it is too long.
c
              IF (LNBLNK(cmdarg) .GT. maxca) THEN
                  istat = -99
                  RETURN
              END IF
              narg = narg + 1
              arg(narg) = cmdarg(1:LNBLNK(cmdarg))
              flg(narg) = 'NONE'
              noflgf    = noflgf + 1
c
          END IF
c
      END IF
c
c Go to argument count checks.
c
      GO TO 99
c
      END
