      SUBROUTINE f7open(lu, file, status, io, form, ier)
c
c Machine-dependent front end for Fortran OPEN.
c
c File f7open.F (Version 1.4).  Last modified at 15:43:48 on 1/23/95.
c
c From:   National Simulation Resource Facility
c         Center for Bioengineering (WD-12)
c         University of Washington
c         Seattle, WA  98195
c
c         Dr. J. Bassingthwaighte, Director
c
c
c Copyright (C) 1989-1995 by
c National Simulation Resource, Univ of WA. All Rights Reserved.
c Software may be copied so long as this copyright notice is included.
c
c This software was developed with support from NIH grant RR-01243.
c Please cite this grant in any publication for which this software
c is used and send one reprint to the address given above.
c
c.......................................................................
c
c DESCRIPTION:
c     This routine provides a machine-independent front-end to the
c     Fortran77 OPEN statement, which is not sufficiently standardized
c     for portable applications. It is expected that this routine will
c     have to be customized for each system.
c
c
c LIMITATIONS:
c     1. This routine will only open sequentially accessed files.
c     2. Not all features (eg. readonly files) are supported on all
c        systems.
c
c CALL AND PARAMETER TYPES:
c     INTEGER lu, ier
c     CHARACTER*(*) file, status, io, form
c     CALL f7open(lu, file, status, io, form, ier)
c
c INPUTS:
c     lu     = logical unit number.
c              If lu<0, a free logical unit is found and the lu is
c              returned.
c              If lu>0, lu must be less than LUMAX (defined below).
c              WARNING! If lu<0, lu must be a variable.
c     file   = name of the file to be opened. If blank, a legal file
c              name will be constructed, but it will not be returned in
c              "file".
c              The name of this file is system-dependent. If the name
c              is needed, obtain it with INQUIRE(lu,name=char-variable)
c              after the file is open.
c     status = approximately equivalent to the STATUS field of the OPEN
c              statement. Upper, lower, or mixed case may be used, but
c              only the first 3 characters are examined.
c                 'OLD'  the file must(!) already exist.
c                 'NEW'  the file must not(!) already exist.
c                 'UNK'  if it exists, open the existing file, else
c                        open a new file. Equivalent to OLD or NEW,
c                        depending on whether the file already exists.
c                 'FRE'  open a fresh file. Delete the old version of
c                        the file, if one exists, before opening as NEW.
c                 'APP'  open an OLD file, positioned at the end.
c                 '   '  default: UNK
c              NOTE: All options other than 'APP' are guaranteed to
c                    position the file at the beginning.
c     io     = intention with regards to reading & writing the file.
c              Only the first 2 characters will be checked, and these
c              may be in any case. Must be one of:
c                 'RO'   readonly. Writing to the file is an error.
c                 'WO'   writeonly. Reading the file is an error.
c                 'RW'   read & write.
c                 '  '   default: RW
c              This argument must be one of the 3 possibilities above,
c              but it is not guaranteed that this option can be
c              implemented on all systems.
c     form   = the FORMATTED option for the OPEN statement. Only the
c              first 3 characters are checked, and these may be in any
c              case. Must be one of the following:
c                 'FOR'  formatted
c                 'UNF'  or 'BIN' unformatted (i.e. binary)
c                 '   '  default: FOR
c
c OUTPUTS:
c     lu     = logical unit number. Also an INPUT (so see above).
c     ier    = returns error flag: 0 = no error; negative = an error.
c
c SUBROUTINES CALLED:
c     igetc     get any available logical unit number
c     f7case    convert a string to proper fortran case (internal)
c
c HISTORY:
c
c Written: 12/04/89       L. Weissman
c
c Modified:
c Ver. 1.3: Added DG version. (OCT93 - R.B. King)
c           Minor fixes. Changed label SUN to sun and label DGUX to dgux.
c           Version number not changed. (AUG94 - W. Chan)
c Ver. 1.4: Added OSF/1 version (JAN95 - L. Weissman)
c
c -----------------------------------------------------------------
c
      INTEGER  lu, ier
      CHARACTER*(*) file, status, io, form
c
c  Local variables
c
      INTEGER      LENFIL, NSTAT, NIO, NFORM, LUMAX
      PARAMETER   (LENFIL=64, NSTAT=5, NIO=3, NFORM=3, LUMAX=255)
      INTEGER      kstat, kio, kform, igetc, klu
      LOGICAL      opened
      CHARACTER*3  f7case, s, sstat(NSTAT), sio(NIO), sform(NFORM)
      EXTERNAL     f7case, igetc
      CHARACTER    sfile*(LENFIL), sbin*11
#ifdef dgux
      CHARACTER*6  intent(NIO)
#endif
      CHARACTER*64 sid1,sid2
c
c  Source control code strings
c
      DATA sid1/'@(#)f7open.F	1.4 created 1/23/95 15:43:48.'/
      DATA sid2/'@(#) retrieved 10/21/96 14:13:59.'/
c
c  Local DATA
c
      DATA sio  /'RO ','WO ','RW '/
      DATA sstat/'NEW','OLD','UNK','FRE','APP'/
      DATA sform/'FOR','UNF','BIN'/
#ifdef dgux
      DATA intent/'INPUT','OUTPUT','BOTH'/
#endif
c
c  I. Check and decode parameters
c
      ier = -1
      opened = .FALSE.
      IF(lu .GT. LUMAX)RETURN
c
      s = f7case(status)
      IF(s .EQ. ' ')s='UNK'
      DO 10 kstat=1,NSTAT
          IF(s .EQ. sstat(kstat))GO TO 20
   10 CONTINUE
      RETURN
c
   20 s = f7case(io)
      s(3:3) = ' '
      IF(s .EQ. ' ')s='RW'
      DO 30 kio=1,NIO
          IF(s .EQ. sio(kio))GO TO 40
   30 CONTINUE
c                                       'RO' always implies 'OLD'
   40 IF(kio .EQ. 1 .AND. kstat .NE. 2)RETURN
c
      s = f7case(form)
      IF(s .EQ. ' ')s='FOR'
      DO 50 kform=1,NFORM
          IF(s .EQ. sform(kform))GO TO 60
   50 CONTINUE
      RETURN
   60 sbin='formatted'
      IF(kform .EQ. 2 .OR. kform .EQ. 3)
#ifdef dgux
     +    sbin='UNFORMATTED'
#else
     +    sbin='unformatted'
#endif
c
c  II. Provide LU and file name, if necessary
c
      klu = lu
      IF(lu .LT. 0)THEN
          klu = igetc()
          IF(klu .LT. 0)RETURN
      ENDIF
c
      IF(file .EQ. ' ')THEN
          sfile=' '
          WRITE(sfile,65)klu
   65     FORMAT('fortran.',I3.3)
      ELSE
          sfile = file
      ENDIF
c
c  III. Open the file. This is all system dependent.
c
#ifdef dgux
c
c  Data General UNIX only
c
      IF(kstat .EQ. 1)THEN
          OPEN(klu,FILE=sfile,STATUS='NEW',FORM=sbin,ERR=100,
     +        PAD='YES',IOINTENT=intent(kio),POSITION='START')
      ELSEIF(kstat .EQ. 2)THEN
          OPEN(klu,FILE=sfile,STATUS='OLD',FORM=sbin,ERR=100,
     +        PAD='YES',IOINTENT=intent(kio),POSITION='START')
      ELSEIF(kstat .EQ. 3)THEN
          OPEN(klu,FILE=sfile,STATUS='UNKNOWN',FORM=sbin,ERR=100,
     +        PAD='YES',IOINTENT=intent(kio),POSITION='START')
      ELSEIF(kstat .EQ. 4)THEN
          OPEN(klu,FILE=sfile,STATUS='FRESH',FORM=sbin,ERR=100,
     +        PAD='YES',IOINTENT=intent(kio),POSITION='START')
      ELSEIF(kstat .EQ. 5)THEN
          OPEN(klu,FILE=sfile,STATUS='OLD',FORM=sbin,ERR=100,
     +        PAD='YES',IOINTENT=intent(kio),POSITION='END')
#else /* !dgux */
c
c ** Systems derived from BSD portable f77, Sun, DEC **
c
      IF(kstat .EQ. 1)THEN
          OPEN(klu,FILE=sfile,STATUS='new',FORM=sbin,ERR=100)
      ELSEIF(kstat .EQ. 2)THEN
#ifdef __alpha
c
c  IV. Some systems honor READONLY
c
          IF(kio .EQ. 1)THEN
              OPEN(klu,FILE=sfile,STATUS='old',READONLY,
     +            FORM=sbin,ERR=100)
          ELSE
              OPEN(klu,FILE=sfile,STATUS='old',FORM=sbin,ERR=100)
          ENDIF
#else
          OPEN(klu,FILE=sfile,STATUS='old',FORM=sbin,ERR=100)
#endif
      ELSEIF(kstat .EQ. 3)THEN
          OPEN(klu,FILE=sfile,STATUS='unknown',FORM=sbin,ERR=100)
      ELSEIF(kstat .EQ. 4)THEN
          OPEN(klu,FILE=sfile,STATUS='old',FORM=sbin,ERR=75)
          CLOSE(klu,STATUS='delete',ERR=100)
   75     OPEN(klu,FILE=sfile,STATUS='new',FORM=sbin,ERR=100)
      ELSEIF(kstat .EQ. 5)THEN
          OPEN(klu,FILE=sfile,STATUS='old',FORM=sbin,ERR=100)
          REWIND klu
          opened = .TRUE.
   80     IF(kform .EQ. 1)THEN
              READ(klu,85,END=90,ERR=100)
   85         FORMAT()
          ELSE
              READ(klu,END=90,ERR=100)
          ENDIF
          GO TO 80
#endif
      ELSE
          RETURN
      ENDIF
      REWIND klu
c
c  V. Returns - normal and otherwise
c
   90 IF(lu .LT. 0)lu = klu
      ier = 0
      RETURN
  100 IF(opened)CLOSE(klu,ERR=110)
  110 RETURN
      END
