      PROGRAM hurst
c
c Hurst analysis
c
c @(#)hurst.f	1.4  :  Last modified on 11/30/01 at 16:40:13.
c
c.......................................................................
c
c From:   National Simulation Resource
c         Center for Bioengineering  (WD-12)
c         University of Washington
c         Seattle, WA 98195
c
c         Dr. J. B. Bassingthwaighte, Director
c
c.......................................................................
c Copyright (C) 1994 - 1995
c National Simulation Resource, Center for Bioengineering,
c University of Washington, Seattle, WA 98195.
c All rights reserved.
c
c This software may be copied so long as the following copyright notice
c is included:
c
c "This software was developed with support from NIH grant RR-01243"
c
c Please cite this grant in publications for which the software or
c derivatives from it were used and send one reprint to the address
c given above.
c.......................................................................
c
c DESCRIPTION
c
c hurst analysis on a time series
c
c hurst [-c ncol] [-t trend] [-s skip] [input_file_name]
c 
c hurst calculates an estimate of the Hurst coefficient us-
c ing rescaled range analysis of the time series contained
c in the input file. The program creates seven output
c files. The input file is named on the command line and is
c given as input_file_name. If the input_file_name is miss-
c ing, the input is assumed to come from standard input.
c
c Let x(j), j=1,2,...n be a series of N points. Subtract the
c mean from x so that the new mean is zero. Over some por-
c tion of this data from j=i,...i+kmax, define
c 
c VN(i,k,kmax) = sum from j=i to i+k (x(j)), k=0 to kmax.
c
c The mean from j=i to i+kmax is given by
c
c Mean(i,kmax)=VN(i,kmax,kmax)/(kmax+1).
c
c Define
c
c VR(i,k,kmax) = sum from j=i to i+k ( x(j)-Mean(i,kmax) ),
c                k=0 to kmax.
c
c The difference between the maximum and minimum
c values of VN(i,k,kmax) as k varies from 0 to kmax is the
c Range, R(i,kmax). The difference between the maximum and
c minimum values of VR(i,k,kmax) as k varies from 0 to kmax
c is also known as the Range, R(i,kmax). The range of VR is
c called the trend corrected range, because
c VR(i,kmax,kmax)=0.0. The range, R(i,kmax), in either case
c is normalized by the standard deviation of the data, S,
c over the same x values. We speak of the rescaled range
c without trend correction, or with trend correction,
c respectively.
c 
c The Rescaled Range (R/S) is computed for the entire
c series. The series is then halved. Each halved series has
c its R/S calculated. If the original series contained an
c odd number of points, the odd point is counted in the mid-
c dle of the series and belongs to both half-length series.
c In this fashion the series continues to be subdivided un-
c til the final pieces have at least six points. The res-
c caled range is computed for all pieces of the series. The
c length (the number of points) of each piece is normalized
c by the total length of the series.
c 
c The analysis proceeds along three distinct paths:
c the analysis of all the data (multipox analysis),
c the means of the rescaled ranges for each particular
c length (mean analysis), and a subset of the data (pox
c analysis). In each case the Y-on-X regression is calcu-
c lated.
c
c In multipox analysis the regression is between the loga-
c rithms of the normalized lengths and the logarithms of the
c rescaled ranges.
c
c In mean analysis, the regression is between the logarithms
c of the normalized lengths and the logarithms of the means
c of the rescaled ranges for those lengths.
c
c In pox analysis, the regression is between the logarithms
c of the normalized lengths and the logaritms of a maximum
c of ten values of the rescaled range at each normalized
c length.
c
c The slope of these regressions are the estimated Hurst
c coefficients.
c 
c The seven output files are named:
c 
c input_file_name.hurst.multipox
c                                     Contains 2 columns of data. The
c                                     first column contains the normal-
c                                     ized length, and the second column
c                                     contains the rescaled range (R/S).
c input_file_name.hurst.mean
c                                     Contains 6 columns of data. Column
c                                     one is the normalized length. For
c                                     each length, the subsequent columns
c                                     are: column two, the average of the
c                                     R/S; column three, the standard de-
c                                     viation of the R/S; column four,
c                                     the standard error of the R/S;
c                                     column five, the minimum value of
c                                     the R/S; and column six, the max-
c                                     imum value of the R/S.
c input_file_name.hurst.pox
c                                     Contains 2 columns of data. The
c                                     first column contains the normal-
c                                     ized. The second column contains
c                                     the R/S for that length. For each
c                                     length, a maximum of ten values are
c                                     used.
c input_file_name.hurst.multipox.line
c                                     Contains 2 columns of data and is
c                                     the linear fit through the loga-
c                                     rithms of the data in
c                                     input_file_name.hurst.multipox.
c input_file_name.hurst.mean.line
c                                     Contains 2 columns of data and is
c                                     the linear fit through the loga-
c                                     rithms of the data in
c                                     input_file_name.hurst.mean.
c input_file_name.hurst.pox.line
c                                     Contains 2 columns of data and is
c                                     the linear fit through the loga-
c                                     rithms of the data in
c                                     input_file_name.hurst.pox.
c input_file_name.hurst.summary.fits
c                                     Summarizes the analysis and the
c                                     three fits, and gives the estimated
c                                     Hurst coefficient for all three
c                                     methods of analysis.
c 
c 
c FLAGS/ARGUMENTS
c 
c -c
c                 Ncol is the number of columns in the output files. If
c                 the number of columns is 2, the first column contains
c                 consecutive integers, and the second column contains
c                 the data. The default is 1.
c -t
c                 If trend equals r, VR is used to calculate the range.
c                 If trend equals n, VN is used to calculate the range.
c                 The default is "n".
c -s
c                 Skip is the number of largest bin sizes to skip when
c                 fitting straight lines to the logarithms of the data in
c                 all three analyses. For example, if a value of 3 were
c                 to be used, the R/S values for normalized lengths=1.0,
c                 0.5, and 0.25 would be omitted from the line fitting.
c 
c                 For all the data and the pox analysis this removes 7
c                 values (one value for normalized lengths =1, two values
c                 for length=0.5, and four values for length=0.25). For
c                 the mean data, it would remove 3 values (one value for
c                 normalized lengths = 1.0, 0.5, and 0.25). The reason
c                 for removing these values is that they lack statistical
c                 significance and tend to be biased. The default is 0.
c input_file_name
c                 If the input file name is missing, the input is read
c                 from standard input, and the output file names will be
c                 hurst.multipox, hurst.mean, etc.
c 
c EXAMPLE
c 
c A file named "data" contains two columns. The
c first column is time and the second column is a measured
c signal. The Hurst analysis is to be done without trend re-
c moval. All the rescaled ranges will be used to determine
c the Hurst coefficient. Enter the following command:
c hurst -c 2 -t n -s 0 data
c The summary of the analysis will be found in the file,
c "data.hurst.summary.fits."  
c
c WARNINGS/LIMITATIONS
c Maximum number of points in a file processed equals 1,048,576, 
c excess points are ignored. Minimum number of points in a
c file is 8. 
c 
c DIAGNOSTICS
c 
c If the program fails because of improper arguments ,
c a diagnostic message indicating the cause of the failure 
c is written to stderr.     
c 
c REFERENCES:
c
c Evaluating Rescaled Range Analysis
c for Time Series, J.B. Bassingthwaighte and G M. Raymond,
c Annals of Biomedical Engineering, Vol. 22, pp. 432-444,
c 1994.  
c 
c SUBROUTINES/FUNCTIONS CALLED
c
c   Internal:
c     calrsr - Calculate rescaled range for a series.
c
c   NSR Math Library:
c     f7args - Process command line arguments and switches
c     f7init - Initialize FORTRAN program
c     f7open - Open FORTRAN sequential files
c     f7clos - Close FORTRAN sequential files
c     f7exit - Exit FORTRAN program
c     linreg - Weighted geometric mean linear regression
c
c   Combined Math Library:
c     r1mach - Machine dependent constants
c
c HISTORY:
c
c Written:  G Raymond (NOV94)
c
c Modified:
c Ver. 1.2:  Changed flg and arg elements from 'none' to 'NONE'.
c            (G Raymond - JUL95)
c Ver. 1.3:  Minor typo corrections. (G. Raymond - OCT97)
c Ver. 1.4
c     Revised:
c       11/27/01    
c       Revision from Dr. Jerry Trzeciakowski:
c       One statement was removed and replaced.
c       tr(inewst+1,2)=calrsr(x(mid),length,itrend)   REPLACED WITH
c       tr(inewst+1,2)=calrsr(x(mid+1),length,itrend)
c       The original statement was incorrect and yielded
c       incorrect results. There were two changes made
c       in husubs.f.
c       The analysis contained in the reference was redone.
c       The conclusions remain fundamentally unchanged.
c        --Gary Raymond
c
c
c-----------------------------------------------------------------------
c
c Define variables
c
      INTEGER NLOG2, NMAX, NBIN, ncnt
      PARAMETER (NLOG2=20, NMAX=2**NLOG2, NBIN=2**(NLOG2-1))
      REAL readi, readx, x(NMAX),tr(NBIN,2)
      INTEGER m(NBIN,2), iarg, icol, itrend
      INTEGER mid, istart,iend, ier
      REAL trmean(NLOG2,2), trsdev(NLOG2),trster(NLOG2), trcnt(NLOG2)
      REAL trmax(NLOG2),trmin(NLOG2)    
      REAL trpox(NLOG2*10,2)
      REAL trfita(NLOG2),trfitm(NLOG2),trfitp(NLOG2)
      REAL stata(8),statm(8),statp(8)
      REAL slopea,slopem,slopep
      REAL yinta, yintm, yintp
      INTEGER istrta,ienda,ilena,istrtm,iendm,ilenm,istrtp,iendp,ilenp
      REAL delt,xolden,xoldst,xp
      INTEGER ioldst,inewst,iold,iolden,ipox,ipoxst,iskpbn
      INTEGER i,ip,jp,istat,mlog2,length
      LOGICAL even
      REAL calrsr,r1mach
      INTEGER lnblnk
      EXTERNAL calrsr,r1mach
      REAL avgx
c 
c Argument variables
c
      INTEGER LUERR
      PARAMETER( LUERR=0)
      CHARACTER*80 infil,outfil
      CHARACTER*80 filnam
      INTEGER luin,luout
      CHARACTER*1 ctrend
      INTEGER       MAXARG,  narg
      PARAMETER    (MAXARG=4)
      INTEGER STDIN
      PARAMETER (STDIN=5)
      CHARACTER*4   flgi(MAXARG), flga(MAXARG)
      CHARACTER*60  arg(MAXARG),  flg(MAXARG)
      CHARACTER*64  SCCSID
      DATA flgi/'-c','-t','-s','NONE'/
      DATA flga/'    ','    ','    ','    '/
      DATA icol, itrend, iskpbn, luin / 1, 0, 0, -1 /
      DATA ctrend /'n'/
      DATA infil,outfil /' ',' '/
c
      DATA          SCCSID
     + /'@(#)hurst.f	1.4  :  Last modified on 11/30/01 at 16:40:13.'/
c
c  This is a statement function
c
      even(length) = MOD(length,2) .EQ. 0
c
c  Process command line entries
c
      CALL f7init()
      CALL f7args(MAXARG, flgi, flga, narg, arg, flg, istat)
      IF (istat .LT. 0) THEN
          WRITE(LUERR, 1)
    1     FORMAT('Usage: hurst [-c ncol] [-t trend] [-s skip] ',
     +                         'input_file_name',/,
     +           '       ncol:    number of columns in the data ',
     +                            '1 (data only) or 2 (line & data)',/,
     +           '       trend:   n (subtract data mean) r (subtract ',
     +                           'local mean)',/,
     +           '       skip:    number of the largest normalized ',
     +                           'lengths to ignore',/,
     +           '       input_file-name: name of input data file')
          CALL f7exit
      END IF
c
c Check for which column to analyze, whether or not to remove trend,
c number of bins to skip in log-log line fits, and input file name
c
      DO 3 iarg = 1, narg
          IF (flg(iarg)(1:2) .EQ. '-c') THEN
              READ(arg(iarg),*,ERR=3) icol
              icol=MAX(1,icol)
              icol=MIN(2,icol)
c
          ELSE IF (flg(iarg)(1:2) .EQ. '-t') THEN
              IF((arg(iarg)(1:1).EQ.'r')
     +            .OR.(arg(iarg)(1:1).EQ.'R')) THEN
                  itrend=1
                  ctrend='r'
              ENDIF
c
          ELSE IF (flg(iarg)(1:2) .EQ. '-s') THEN
              READ(arg(iarg),*,ERR=3) iskpbn
c
          ELSE IF (flg(iarg)(1:4) .EQ. 'NONE') THEN
              infil=arg(iarg)(:LNBLNK(arg(iarg)))
              IF(LNBLNK(arg(iarg)).NE.0) THEN
                    CALL f7open (luin, infil,
     +                     'OLD', 'RO', 'FOR', ier)
                    IF(luin.EQ.-1) THEN
                        WRITE(LUERR,2)
    2                   FORMAT('Input file missing.')
                        CALL f7exit()
                    ENDIF
              END IF
          ENDIF
    3 CONTINUE
c
c Read data
c
      IF(luin.EQ.-1) luin=STDIN
      outfil=infil(:LNBLNK(infil))//'.hurst.'
      IF(luin.EQ.STDIN) THEN
          outfil= 'hurst.'
          infil= 'STANDARD INPUT'
      ENDIF
      ncnt=0
      IF(icol.EQ.1) THEN
          DO 10 i=1, NMAX
              READ(luin,*,END=19,ERR=991) readx
              ncnt=ncnt+1
              x(ncnt)=readx
   10     CONTINUE
      ELSE
          DO 11 i=1, NMAX
              READ(luin,*,END=19,ERR=991) readi,readx
              ncnt=ncnt+1
              x(ncnt)=readx
   11     CONTINUE 
      ENDIF
   19 CONTINUE
      CALL f7clos(luin,'KEE',ier)
c
c The rescaled range analysis on series of length ncnt
c
      IF(ncnt.LT.8) THEN
          WRITE(LUERR,41) ncnt
   41     FORMAT('Only ',I3,' data items in file.  Minimum is 8 for ',
     +      'analysis.')
          CALL f7exit()
      END IF
c
c If local mean not being removed, set subtract global mean.
c
      IF(itrend.EQ.0) THEN
          avgx=0.0
          DO 20 i=1,ncnt
              avgx=avgx +x(i)
   20     CONTINUE
          avgx=avgx/REAL(ncnt)
          DO 21 i=1,ncnt
              x(i)=x(i)-avgx
   21     CONTINUE
      ENDIF
c
      m(1,1)=1
      m(1,2)=ncnt
      length = m(1,2)-m(1,1)+1
      tr(1,1) = REAL(length)/REAL(ncnt)
      tr(1,2)= calrsr(x(m(1,1)),length,itrend)
      DO 1000 i=2,NLOG2+1
          ioldst=2**(i-2)
          iolden=ioldst+2**(i-2) -1
          inewst=2**(i-1)
          DO 100 iold=ioldst,iolden
              length = m(iold,2)-m(iold,1)+1
              IF(length.LT.7) THEN
                  mlog2=i-1
                  GOTO 1001
              ENDIF
              istart=m(iold,1)
              iend=m(iold,2)
              IF(EVEN(length)) THEN
                  mid=(istart+iend)/2
                  m(inewst,1)=istart
                  m(inewst,2)=mid
                  length=mid-istart+1
                  tr(inewst,1)=REAL(length)/REAL(ncnt)
                  tr(inewst,2)=calrsr(x(istart),length,itrend)
                  m(inewst+1,1)=mid+1
                  m(inewst+1,2)=iend
                  tr(inewst+1,1)=tr(inewst,1)
                  tr(inewst+1,2)=calrsr(x(mid+1),length,itrend)
                  inewst=inewst+2
              ELSE
                  mid=(istart+iend)/2
                  m(inewst,1)=istart
                  m(inewst,2)=mid
                  length=mid-istart+1
                  tr(inewst,1)=REAL(length)/REAL(ncnt)
                  tr(inewst,2)=calrsr(x(istart),length,itrend)
                  m(inewst+1,1)=mid
                  m(inewst+1,2)=iend
                  tr(inewst+1,1)=tr(inewst,1)
                  tr(inewst+1,2)=calrsr(x(mid),length,itrend)
                  inewst=inewst+2
              ENDIF
  100     CONTINUE
 1000 CONTINUE
 1001 CONTINUE
c
c Compute mean array
c
      DO 2200 i=1,mlog2
          ioldst=2**(i-1)
          iolden=ioldst+2**(i-1) -1
          trmean(i,1)=0.0
          trmean(i,2)=0.0
          trsdev(i)=0.0
          trster(i)=0.0
          trcnt(i)=0.0
          trmin(i)=r1mach(2)
          trmax(i)=-trmin(i)
          DO 2100 iold=ioldst,iolden,1
              trmean(i,1)=trmean(i,1)+tr(iold,1)
              trmean(i,2)=trmean(i,2)+tr(iold,2)
              trcnt(i)=trcnt(i)+1.
              trsdev(i)=trsdev(i)+tr(iold,2)*tr(iold,2)
              trmin(i)=MIN(trmin(i),tr(iold,2))
              trmax(i)=MAX(trmax(i),tr(iold,2))
 2100     CONTINUE
          IF(trcnt(i).GT.1) THEN
              trmean(i,1)=trmean(i,1)/trcnt(i)
              trmean(i,2)=trmean(i,2)/trcnt(i)
              trsdev(i)=SQRT(ABS((trsdev(i)
     +                        -trcnt(i)*trmean(i,2)*trmean(i,2))
     +                        /(trcnt(i)-1.)))
              trster(i)=trsdev(i)/trcnt(i)
          ELSE
              trsdev(i)=0.0
              trster(i)=0.0
          ENDIF
 2200 CONTINUE
c
c Compute pox array (max of ten from each group)
c
      ipox=0
      DO 3200 i=1,mLOG2
          ioldst=2**(i-1)
          iolden=ioldst+2**(i-1) -1
          length=iolden-ioldst+1
          IF(length.le.10) THEN
              DO 3100 ip=ioldst,iolden
                  ipox=ipox+1
                  trpox(ipox,1)=tr(ip,1)
                  trpox(ipox,2)=tr(ip,2)
 3100         CONTINUE
          ELSE
              xoldst=REAL(ioldst)
              xolden=REAL(iolden)
              delt=(xolden-xoldst+1.0)/10.
              xp=xoldst+delt/4.-delt
              DO 3150 jp=1,10
                 ipox=ipox+1
                 xp=xp+delt
                 ip=NINT(xp)
                 trpox(ipox,1)=tr(ip,1)
                 trpox(ipox,2)=tr(ip,2)
 3150         CONTINUE
          ENDIF
 3200 CONTINUE
c
c Compute log-log fits for all data, excluding the iskpbn bins
c
      iskpbn=MIN(iskpbn,mlog2-2)
      iskpbn=MAX(0,iskpbn)
      istrta=2**iskpbn
      ienda=2**mlog2 -1
      ilena=ienda-istrta+1
      DO 4100 i=istrta,ienda
          tr(i,1)=LOG(tr(i,1))
          tr(i,2)=LOG(tr(i,2))
 4100 CONTINUE
      CALL linreg(tr(istrta,1),tr(istrta,2),ilena,1.0,
     +            slopea,yinta,stata,ier)
      yinta=EXP(yinta)
      DO 4200 i=istrta,ienda
          tr(i,1)=EXP(tr(i,1))
          tr(i,2)=EXP(tr(i,2))
 4200 CONTINUE
c
c Compute log-log fit for the mean of the data, excluding the
c iskpbn points
c
      istrtm=iskpbn+1
      iendm=mlog2
      ilenm=iendm-istrtm+1
      DO 4300 i=istrtm,iendm
          trmean(i,1)=LOG(trmean(i,1))
          trmean(i,2)=LOG(trmean(i,2))
 4300 CONTINUE
      CALL linreg(trmean(istrtm,1),trmean(istrtm,2),ilenm,1.0,
     +            slopem,yintm,statm,ier)
      yintm=EXP(yintm)
      DO 4400 i=istrtm,iendm  
          trmean(i,1)=EXP(trmean(i,1))
          trmean(i,2)=EXP(trmean(i,2))
 4400 CONTINUE
c
c Compute log-log fit for the pox data, excluding the pox points
c from the iskpbn bins
c
      IF(iskpbn.LE.4) THEN
          ipoxst=2**iskpbn
      ELSE
          ipoxst=16+(iskpbn-4)*10
      ENDIF
      istrtp=ipoxst
      iendp=ipox
      ilenp=iendp-istrtp+1
      DO 4500 i=istrtp,iendp
          trpox(i,1)=LOG(trpox(i,1))
          trpox(i,2)=LOG(trpox(i,2))
 4500 CONTINUE
      CALL linreg(trpox(istrtp,1),trpox(istrtp,2),ilenp,1.0,
     +            slopep,yintp,statp,ier)
      yintp=EXP(yintp)
      DO 4600 i=istrtp,iendp
          trpox(i,1)=EXP(trpox(i,1))
          trpox(i,2)=EXP(trpox(i,2))
 4600 CONTINUE
c
c Compute the fitted lines over the means
c
      DO 4700 i=1,mlog2
          trfita(i)=yinta*(trmean(i,1)**slopea)
          trfitm(i)=yintm*(trmean(i,1)**slopem)
          trfitp(i)=yintp*(trmean(i,1)**slopep)
 4700 CONTINUE
c
c Write out all data analysis results
c
      filnam=outfil(:LNBLNK(outfil))//'multipox'
      luout=-1
      CALL f7open(luout,filnam,'UNK','WO','FOR',ier)
      iend=2**mlog2-1
      DO 5100 i=1,iend
          WRITE(luout,*) tr(i,1),tr(i,2)
 5100 CONTINUE
      CALL f7clos(luout,'KEE',ier)
c---------------
      filnam=outfil(:LNBLNK(outfil))//'mean'
      luout=-1
      CALL f7open(luout,filnam,'UNK','WO','FOR',ier)
      DO 5200 i=1,mlog2
          WRITE(luout,*) trmean(i,1),trmean(i,2),trsdev(i),
     +                   trster(i),trmin(i),trmax(i)
 5200 CONTINUE
      CALL f7clos(luout,'KEE',ier)
c---------------
      filnam=outfil(:LNBLNK(outfil))//'pox'
      luout=-1
      CALL f7open(luout,filnam,'UNK','WO','FOR',ier)
      DO 5300 i=1,ipox 
          WRITE(luout,*) trpox(i,1),trpox(i,2)
 5300 CONTINUE
      CALL f7clos(luout,'KEE',ier)
c---------------
      filnam=outfil(:LNBLNK(outfil))//'multipox.line'
      luout=-1
      CALL f7open(luout,filnam,'UNK','WO','FOR',ier)
      DO 5400 i=1,mlog2
          WRITE(luout,*) trmean(i,1),trfita(i)
 5400 CONTINUE
      CALL f7clos(luout,'KEE',ier)
c---------------
      filnam=outfil(:LNBLNK(outfil))//'mean.line'
      luout=-1
      CALL f7open(luout,filnam,'UNK','WO','FOR',ier)
      DO 5500 i=1,mlog2
          WRITE(luout,*) trmean(i,1),trfitm(i)
 5500 CONTINUE
      CALL f7clos(luout,'KEE',ier)
c---------------
      filnam=outfil(:LNBLNK(outfil))//'pox.line'
      luout=-1
      CALL f7open(luout,filnam,'UNK','WO','FOR',ier)
      DO 5600 i=1,mlog2
          WRITE(luout,*) trmean(i,1),trfitp(i)
 5600 CONTINUE
      CALL f7clos(luout,'KEE',ier)
c---------------
      filnam=outfil(:LNBLNK(outfil))//'summary.fits'
      luout=-1
      CALL f7open(luout,filnam,'UNK','WO','FOR',ier)
      WRITE(luout,5701) infil
 5701 FORMAT('Hurst time series analysis',/,
     +       ' ',/,
     +       'Input data file :',A,/,
     +       ' ')
      WRITE(luout,5703) icol
 5703 FORMAT('Input data column is ',I2)
      IF(ctrend.EQ.'r') THEN
          WRITE(luout,5705)
 5705     FORMAT('Trend corrected: Local mean subtracted.')   
      ELSE
          WRITE(luout,5706) 
 5706     FORMAT('Not trend corrected: Global mean subtracted.')
      ENDIF
      WRITE(luout,5709) iskpbn
 5709 FORMAT('Number of bins skipped in line fits is ',I3,/,' ',/,' ')
      WRITE(luout,5711) 'multipox',yinta,slopea,stata(3),
     +                  istrta,ienda,ilena,'multipox',slopea
 5711 FORMAT(A,' data log-log fit',/,
     +       'R/S(t/tmax)= ',E13.5,'*(t/tmax)**',E13.5,/,
     +       '  correlation coefficient:',E12.4,/,
     +       '  starting point  : ',I6,/,
     +       '  ending  point   : ',I6,/,
     +       '  number of points: ',I6,/,
     +       '  ',A,' estimated Hurst coefficient',E13.5)

      WRITE(luout,5711) 'mean',yintm,slopem,statm(3),
     +                  istrtm,iendm,ilenm,'mean',slopem
      WRITE(luout,5711) 'pox',yintp,slopep,statp(3),
     +                  istrtp,iendp,ilenp,'pox',slopep
      CALL f7clos(luout,'KEE',ier)
      CALL f7exit()
c
  991 WRITE(LUERR,992) ncnt+1
  992 FORMAT('Error in reading input on line ',I7)
      Call f7exit()
      END
