      REAL FUNCTION calrsr(x,n,itrend)
c
c Calculate Rescaled range of the numbers contained in the vector x(n)
c
c @(#)husubs.f	1.2  :  Last modified on 11/30/01 at 16:40:40.
c
c.......................................................................
c
c From:   National Simulation Resource
c         Center for Bioengineering  (WD-12)
c         University of Washington
c         Seattle, WA 98195
c
c         Dr. J. B. Bassingthwaighte, Director
c
c.......................................................................
c Copyright (C) 1994
c National Simulation Resource, Center for Bioengineering,
c University of Washington, Seattle, WA 98195.
c All rights reserved.
c
c This software may be copied so long as the following copyright notice
c is included:
c
c "This software was developed with support from NIH grant RR-01243"
c
c Please cite this grant in publications for which the software or
c derivatives from it were used and send one reprint to the address
c given above.
c.......................................................................
c
c DESCRIPTION:
c
c Set itrend=1  (Subtract the local mean from each segment.)
c   (trend correction)
c
c Set itrend=0 (Do not subtract the local mean for each segment.)
c   (no trend correction)
c
c Given a vector of n numbers, calculate the rescaled range.  The
c steps involved in this calculation are as follows:
c
c 1.) Calculate the mean and standard deviation.
c
c 2.) IF itrend=1, calculate the running sum of (x(i)-xmean) which is 
c     called the range.  Find its minimum and maximum, xmin and xmax,
c     respectively.
c
c     IF itrend=0, calculate the running sum of (x(i)) which is
c     called the range.  Find its minimum and maximum, xmin and xmax,
c     respectively. 
c
c 3.) The rescaled range, calrsr is (xmax-xmin)/(standard deviation).
c
c 4.) In the special case that all the values in x are identical,
c     the rescaled range is defined to be
c             calrsr=(n-1)/SQRT(n)
c     which is the value one gets by perturbing one of the x's.
c     (It is the case that the size of the perturbation is
c      immaterial--it cancels itself from the calculation).
c     (Trend correction has been assumed to get this result.)
c
c CALL AND PARAMETER TYPES
c 
c       INTEGER n, itrend
c       REAL x(n), calrsr, c
c   
c       c=calrsr(x,n,itrend)
c
c Input
c       x(n)    A vector of n elements, n>=2
c       n       The number of elements in x, n>=2
c       itrend  =1 to calculate the running sum of (x(i)-xmean)
c               This makes the final sum = 0.0 Hence this is
c               called trend removal in the analysis
c       itrend  =0 to calculate the running sum of (x(i)).
c               Since the final sum does not in general equal
c               0, this is called no trend removal in the analysis.
c
c Output
c       c, calrsr The rescaled range of the data in the x vector.
c 
c WARNINGS
c
c calrsr returns -1.0 if the number of data points is less
c than two. 
c
c REFERENCES:
c
c Evaluating Rescaled Range Analysis
c for Time Series, J.B. Bassingthwaighte and G M. Raymond,
c Annals of Biomedical Engineering, Vol. 22, pp. 432-444,
c 1994.
c
c FUNCTIONS/SUBROUTINES REQUIRED:
c
c None.
c
c HISTORY:
c
c     Written:
c	11/10/94  ver 1.1  G. Raymond
c     Revised:
c       11/27/01  Ver 1.2  
c       Revision from Dr. Jerry Trzeciakowski
c       The two statements 
c       xmin=0
c       xmax=0 were commented out and replaced with
c       xmin=r1mach(2)
c       xmax=-xmin
c       The original statements were an error and yielded
c       incorrect results. -- G. Raymond
c
c.......................................................................
c
      INTEGER itrend
      REAL x(n)
      INTEGER i,n
      DOUBLE PRECISION dpx,xmean,xmin,xmax,xrange,xstdev,diff
      INTEGER ifac
      REAL afac
c
      CHARACTER*64  SCCSID
      DATA          SCCSID
     + /'@(#)husubs.f	1.2  :  Last modified on 11/30/01 at 16:40:40.'/
c
c Set parameter for trend correction
c
      ifac=itrend
      ifac=MAX(0,ifac)
      ifac=MIN(1,ifac)
      afac=REAL(ifac)
c
      IF(n.LT.2) THEN
          calrsr=-1
      ELSE
          xrange=0.0
c         xmin=0.0
c         xmax=0.0
          xmin=r1mach(2)
          xmax=-xmin
          xmean=0.0
          DO 100 i=1,n
              xmean=xmean+x(i)
  100     CONTINUE
          xmean=xmean/REAL(n)
          xstdev=0.0
          DO 110 i=1,n
              dpx=x(i)
              diff   = dpx-xmean
              xstdev = xstdev+diff*diff
              xrange  = xrange+dpx-afac*xmean
              IF( xrange.LT. xmin) xmin=xrange
              IF( xrange.GT. xmax) xmax=xrange
  110     CONTINUE 
          IF(xstdev.GT.0.0) THEN 
              xstdev=SQRT(xstdev/REAL(n-1))
              calrsr = (xmax-xmin)/xstdev
          ELSE
c All values in x are equal
              calrsr=REAL(n-1)/SQRT(REAL(n))
          ENDIF
      ENDIF
      RETURN
      END
